'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = configure;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

require('./guard');

var _hook = require('./hook');

var _hook2 = _interopRequireDefault(_hook);

var _postcss = require('postcss');

var _postcss2 = _interopRequireDefault(_postcss);

var _path = require('path');

var _fs = require('fs');

var _postcssModulesExtractImports = require('postcss-modules-extract-imports');

var _postcssModulesExtractImports2 = _interopRequireDefault(_postcssModulesExtractImports);

var _postcssModulesLocalByDefault = require('postcss-modules-local-by-default');

var _postcssModulesLocalByDefault2 = _interopRequireDefault(_postcssModulesLocalByDefault);

var _postcssModulesScope = require('postcss-modules-scope');

var _postcssModulesScope2 = _interopRequireDefault(_postcssModulesScope);

var _parser = require('./parser');

var _parser2 = _interopRequireDefault(_parser);

var plugins = [_postcssModulesLocalByDefault2['default'], _postcssModulesExtractImports2['default'], _postcssModulesScope2['default']];
var rootDir = undefined;

/**
 * @param  {string}   sourceString The file content
 * @param  {string}   sourcePath
 * @param  {string}   trace
 * @param  {function} pathFetcher
 * @return {object}
 */
function load(sourceString, sourcePath, trace, pathFetcher) {
  // @todo To think about a better way to export tokens from the plugin
  var exportTokens = {};

  var result = (0, _postcss2['default'])(plugins.concat(new _parser2['default']({ exportTokens: exportTokens, pathFetcher: pathFetcher, trace: trace }))).process(sourceString, { from: '/' + sourcePath }).stringify();

  return { injectableSource: result.css, exportTokens: exportTokens };
}

(0, _hook2['default'])(function (filename) {
  var root = rootDir || (0, _path.dirname)(filename);
  var sources = {};
  var tokensByFile = {};
  var importNr = 0;

  var fetch = function fetch(_newPath, _relativeTo, _trace) {
    var newPath = _newPath.replace(/^["']|["']$/g, '');
    var trace = _trace || String.fromCharCode(importNr++);

    var relativeDir = (0, _path.dirname)(_relativeTo);
    var rootRelativePath = (0, _path.resolve)(relativeDir, newPath);
    var fileRelativePath = (0, _path.resolve)((0, _path.join)(root, relativeDir), newPath);

    var tokens = tokensByFile[fileRelativePath];
    if (tokens) {
      return tokens;
    }

    var source = (0, _fs.readFileSync)(fileRelativePath, 'utf-8');

    var _load = load(source, rootRelativePath, trace, fetch);

    var injectableSource = _load.injectableSource;
    var exportTokens = _load.exportTokens;

    sources[trace] = injectableSource;
    tokensByFile[fileRelativePath] = exportTokens;

    return exportTokens;
  };

  return fetch((0, _path.relative)(root, filename), '/');
});

/**
 * @param  {object} opts
 * @param  {array}  opts.u
 * @param  {array}  opts.use
 */

function configure(opts) {
  opts = opts || {};

  var customPlugins = opts.u || opts.use;
  plugins = Array.isArray(customPlugins) ? customPlugins : [_postcssModulesLocalByDefault2['default'], _postcssModulesExtractImports2['default'], _postcssModulesScope2['default']];

  if (opts.root && typeof opts.root === 'string') {
    rootDir = opts.root;
  }
}

module.exports = exports['default'];