'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _postcss = require('postcss');

var importRegexp = /^:import\((.+)\)$/;

exports['default'] = (0, _postcss.plugin)('parser', function (opts) {
  opts = opts || {};

  // have to export it outside of the plugin :()
  var exportTokens = opts.exportTokens;
  var translations = {};

  var fetchImport = function fetchImport(importNode, relativeTo, depNr) {
    var file = importNode.selector.match(importRegexp)[1];
    var depTrace = opts.trace + String.fromCharCode(depNr);
    var exports = opts.pathFetcher(file, relativeTo, depTrace);

    importNode.each(function (decl) {
      if (decl.type === 'decl') {
        translations[decl.prop] = exports[decl.value];
      }
    });

    importNode.removeSelf();
  };

  var fetchAllImports = function fetchAllImports(css) {
    var imports = 0;

    css.each(function (node) {
      if (node.type === 'rule' && node.selector.match(importRegexp)) {
        fetchImport(node, css.source.input.from, imports++);
      }
    });
  };

  var linkImportedSymbols = function linkImportedSymbols(css) {
    return css.eachDecl(function (decl) {
      Object.keys(translations).forEach(function (translation) {
        decl.value = decl.value.replace(translation, translations[translation]);
      });
    });
  };

  var handleExport = function handleExport(exportNode) {
    exportNode.each(function (decl) {
      if (decl.type === 'decl') {
        Object.keys(translations).forEach(function (translation) {
          decl.value = decl.value.replace(translation, translations[translation]);
        });

        exportTokens[decl.prop] = decl.value;
      }
    });

    exportNode.removeSelf();
  };

  var extractExports = function extractExports(css) {
    return css.each(function (node) {
      if (node.type === 'rule' && node.selector === ':export') handleExport(node);
    });
  };

  return function (css) {
    fetchAllImports(css);
    linkImportedSymbols(css);
    extractExports(css);
  };
});
module.exports = exports['default'];